function [Mc,Post]=mcmc_sparfac(y,ini,priors,Ind,File)

% modell
% x_t= lam*f_t + xi_t
% phi(L)f_t = eta_t  eta_t ~ N(0,I)
% psi(L)xi_t=e_t     e_t ~ N(0,sigeps)

%indices
% nvars: number of variables
% nobs: number of obs
% nf: number of factors
% psilag: autoregressive order of idiosyncratic component
% philag: autoregressive order of factor dynamics
% it= number of MCMC iterations
% it0: number of burn-in iterations
% rth: save every rth draw
% bf: flag to save posterior moments

%input:
% y        	data matrix: nobs x nvars, variables in columns
%initial values of
% lam		nvars x nf
% phi		nf x nf*philag
% psi       nvars x psilag
% sigetast    nf x nf
% sigeps    nvars x 1
% beta      nvars x nf
% rho       nf x 1

%output:
% mmc = fix(it-it0)/rth
% MCMC draws
% ftmc     factors  nobs x nf x mmc
% lammc    nvars x nf x mmc
% lamstarmc nvars x nf x mmc
% phimc    nf x nf*philag x mmc
% psimc    nvars x psilag x mmc
% sigepsmc nvars x 1 x mmc
% betamc   nvars x nf x mmc
% rhomc    nf x 1 x mmc

% define MCMC output
it=Ind.it;
it0=Ind.it0;
rth=Ind.rth;
psilag=Ind.psilag;
philag=Ind.philag;
nf=Ind.nf;
nvars=Ind.nvars;
prior=Ind.prior;
utz=Ind.utz;
nobs=Ind.nobs;
f_ind=Ind.f_ind;
sperm=Ind.sperm;
perm=Ind.perm;
norm=Ind.norm;
bf=Ind.bf;

lamspar=ini.lamspar;
lam=ini.lam;
phistar=ini.phistar;
phi=ini.phi;
psi=ini.psi;
sigeps=ini.sigeps;
sigetast=ini.sigetast;
beta=ini.beta;
tau=ini.tau;
rho=ini.rho;
H=ini.H;
HI=ini.HI;
ftstar=ini.ftstar;
ft=ini.ft;

ind=it0+1:rth:it;
ind_post=[];
if bf
    int=ceil(length(ind)/Ind.bfm);
    ind_post=ind(1:int:end);
end
Mb=length(ind_post);
MMC=length(ind);%1000;%length(ind);
mci=1;
mcip=1;
ftnobs=nobs-psilag;

Mc.ftstar=zeros(ftnobs,nf,MMC);
Mc.lamspar=sparse(zeros(nvars,nf*MMC));
Mc.phistar=zeros(nf,nf*philag,MMC);
Mc.psi=zeros(nvars,psilag,MMC);
Mc.sigeps=zeros(nvars,1,MMC);
Mc.tau=zeros(nf,1,MMC);
Mc.rho=zeros(nf,1,MMC*(prior>0));
Mc.beta=zeros(nvars,nf,MMC*(prior==2));
Mc.loglik=zeros(MMC,1); %may be used to get the marginal likelihood from simstate_chanjel.m (needs to be implemented, though)

Post.lammpost=zeros(nvars,nf,bf*Mb);
Post.lamvpost=zeros(nvars,nf,bf*Mb);
Post.lamoddpost=zeros(nvars,nf,bf*Mb);
Post.sigepspost=zeros(nvars,2,bf*Mb);
Post.rhopost=zeros(nf,2,bf*Mb*(prior>0));
Post.taupost=zeros(nf,2,bf*Mb);
if psilag
    Post.psimpost=zeros(psilag,nvars,bf*Mb);
    Post.psivpost=zeros(psilag*(psilag+1)/2,nvars,bf*Mb);
end
if philag
    Post.phimpost=zeros(nf*nf*philag,1,bf*Mb);
    Post.phivpost=zeros(nf^2*philag*(nf^2*philag+1)/2,1,bf*Mb);
end


acc=0;

factors=1:nf;
loglik=0;
for mmc=1:it;
    if any(mmc==(1:500:it));
        mmc
    end
    % condensed kalman filter, give data y,lam,sigeps,sigetast,phi; out:generated ft */
    ystar=y;
    if any(psilag);
        ystar=lagtrans_ind(y,psilag,psi); %transformed data for condensed state space
    end;
    [ftnew,~,~]=simstate_chanjel(ystar,lam,phi,psi,sigeps,sigetast,psilag,philag,nf);
    
    if norm
        ft=ftnew;
        ftstar=(HI*ft')';
    else
        ftstar=ftnew;
    end
    
    if prior
        [lamspar,tau,beta,rho,lammp,lamip,poddp,rhop,taup]=simparlamsparse(ystar,ftstar,lamspar,psi,sigeps,sigetast,tau,rho,psilag,priors.tau0,priors.beta0,priors.rho0,prior,utz);
    elseif ~prior
        [lamspar,tau]=simparlamnormal(ystar,ftstar,lamspar,psi,sigeps,sigetast,tau,psilag,priors.tau0,utz);
    end
    
    % generating theta\lambda
    [ac,phistar,psi,sigeps,phimp,phiip,psimp,psiip,sigepsp]=simparstarexlamda(y,ftstar,lamspar,phistar,psi,sigeps,psilag,philag,f_ind,priors.p0,priors.P0inf,priors.q0,priors.Q0inf,priors.sigeps0);
    
    if it>it0;
        acc=acc+ac;
    end
    
    if ~sperm;
        s_ind=checkpositivity(lamspar);
    elseif sperm;
        s_ind=find(rand(1,nf)>0.5);
    end
    if any(s_ind)
        pid=find(~any(factors(ones(length(s_ind),1),:)'==s_ind(ones(1,nf),:),2))';
        lamspar(:,s_ind)=-lamspar(:,s_ind);
        ftstar(:,s_ind)=-ftstar(:,s_ind);
        if any(pid)
            for l=1:philag
                phistar(pid,(l-1)*nf+s_ind)= -phistar(pid,(l-1)*nf+s_ind);
                phistar(s_ind,pid+(l-1)*nf)= -phistar(s_ind,pid+(l-1)*nf);
            end
        end
    end
    
    if perm
        if perm==1
            si=(1:nf);
            if prior==2
                [~,si]=sort(sum(lamspar~=0,1),'descend');
                beta=beta(:,si);
            elseif prior==1
                [~,si]=sort(sum(lamspar~=0,1),'descend');
            elseif prior==0
                [~,si]=sort(sum(lamspar.^2,1),'descend');
            end
            
        elseif perm==2
            si=randperm(nf);
        end
        lamspar=lamspar(:,si);
        ftstar=ftstar(:,si);
        for j=1:philag;
            phistar(:,(j-1)*nf+1:j*nf)=phistar(si,(j-1)*nf+si);
        end
        if prior
            rho=rho(si);
            if prior==2
                beta=beta(:,si);
            end
        end
        tau=tau(si);
        
    end
    if norm
        [ve,~]=eig(lamspar'*lamspar);
        H=ve(:,end:-1:1)';
        HI=ve(:,end:-1:1);
        lam=lamspar*HI;
        for j=1:philag;
            phi(:,(j-1)*nf+1:j*nf)= H*phistar(:,(j-1)*nf+1:j*nf)*HI;
        end
    elseif ~norm
        lam=lamspar;
        phi=phistar;
    end
    
    % saving
    
    if any(mmc==ind)
        Mc.loglik(mci)=loglik;
        Mc.ftstar(:,:,mci)=ftstar(end-ftnobs+1:end,:);
        Mc.lamspar(:,(mci-1)*nf+1:mci*nf)=sparse(lamspar);
        if philag
            Mc.phistar(:,:,mci)=phistar;
        end
        if psilag
            Mc.psi(:,:,mci)=psi;
        end
        Mc.sigeps(:,:,mci)=sigeps;
        Mc.tau(:,:,mci)=tau;
        if prior;
            Mc.rho(:,:,mci)=rho;
            if prior==2;
                Mc.beta(:,:,mci)=beta;
            end
        end
        mci=mci+1;
    end
    if and(bf,any(mmc==ind_post))
        
        Post.lammpost(:,:,mcip)=lammp;
        Post.lamvpost(:,:,mcip)=lamip;
        Post.lamoddpost(:,:,mcip)=poddp;
        Post.sigepspost(:,:,mcip)=sigepsp;
        Post.taupost(:,:,mcip)=taup;
        if psilag
            Post.psimpost(:,:,mcip)=psimp;
            Post.psivpost(:,:,mcip)=psiip;
        end
        if philag
            Post.phimpost(:,1,mcip)=phimp;
            Post.phivpost(:,1,mcip)=phiip;
        end
        if prior
            Post.rhopost(:,:,mcip)=rhop;
        end
        mcip=mcip+1;
    end
    
    
end



